@@ -2,7 +2,7 @@
  *  html.c
  *  mod_musicindex
  *
- *  $Id: html.c,v 1.75 2004/05/29 13:17:44 varenet Exp $
+ *  $Id: html.c,v 1.89 2004/07/08 18:47:23 boudinr Exp $
  *
  *  Created by Thibaut VARENE on Thu Mar 20 2003.
  *  Copyright (c) 2003-2004 Regis BOUDIN
@@ -21,7 +21,7 @@
  *
  * @author Regis Boudin
  * @author Thibaut Varene
- * @version $Revision: 1.75 $
+ * @version $Revision: 1.89 $
  * @date 2003-2004
  *
  * That file groups all functions related to HTML generation
@@ -32,6 +32,8 @@
  *
  * @todo still some work on the custom table
  * @todo maybe move treat_args somewhere else ?
+ * @todo Replace text with eye-candy icons wherever possible
+ * @todo reduce the size of this file & optimize functions
  */
  
 #include "html.h"
@@ -111,6 +113,10 @@ static void list_songs(request_rec *r, struct mu_ent *p, mu_config *conf)
 				current = "Genre";
 				new = _("Genre");
 				break;
+			case SB_SIZE:
+				current = "Size";
+				new = _("Size");
+				break;
 			default:
 				continue;
 		}
@@ -126,8 +132,6 @@ static void list_songs(request_rec *r, struct mu_ent *p, mu_config *conf)
 		ap_rvputs(r, "   <th class=\"", current, "\">", NULL);
 		if (!(local_options & MI_LISTCUSTOM) && !(conf->search))
 			ap_rprintf(r, "<a href=\"?sort=%c\">%s</a>", conf->fields[i], new);
-		else if ((conf->search) && !(local_options & MI_RECURSIVE))
-			ap_rprintf(r, "<a href=\"?sort=%c&amp;option=%s&amp;action=Search\">%s</a>", conf->fields[i], conf->search, new);
 		else
 			ap_rputs(new, r);
 		ap_rputs("</th>\n", r);
@@ -137,29 +141,37 @@ static void list_songs(request_rec *r, struct mu_ent *p, mu_config *conf)
 	current = "*";
 	new = current;
 
-	while (q) {	/* XXX doc */
+	for (q=p; q!= NULL; q=q->next) {	/* XXX doc */
+		
+		if (q->filetype == FT_DIR)
+			continue;
+		
 		if ((conf->search != NULL) &&
 			(conf->options & (MI_LISTCUSTOM|MI_RECURSIVE)) == MI_RECURSIVE) {
 			new = ap_make_dirstr_parent(r->pool, q->file);
 			
 			if (strcmp(current, new)) {
-				request_rec *subreq = NULL;
 				current = new;
+				ap_rputs("  <tr class=\"title\">\n"
+					"   <th align=\"left\" colspan=\"10\">", r);
 				if ((current[0] == '\0') || (current[1] == '\0')) {
-					ap_rvputs(r, "  <tr class=\"title\"><th align=\"left\" colspan=\"10\">",
-						_("in Current Directory"), "</th>\n", NULL);
+					ap_rputs(_("in Current Directory"), r);
 					local_options = conf->options;	/* apache doesn't know directory '\0' ;P */
 				}
 				else {
-					ap_rvputs(r, "  <tr class=\"title\"><th align=\"left\" colspan=\"10\">in <a href=\"",
-						current, "\">", current, "</a></th>\n", NULL);
-					subreq = ap_sub_req_lookup_uri(current, r, NULL);
-					local_options = ((mu_config*)ap_get_module_config(subreq->per_dir_config, &musicindex_module))->options;
-					ap_destroy_sub_req(subreq);
+					request_rec *subreq = ap_sub_req_lookup_uri(current, r, NULL);
+					ap_rvputs(r, _("in"), "<a href=\"", current, "\">", current, "</a>", NULL);
+					if (subreq) {
+						local_options = ((mu_config*)ap_get_module_config(subreq->per_dir_config, &musicindex_module))->options;
+						ap_destroy_sub_req(subreq);
+					}
+					else
+						local_options = 0;
 				}
+				ap_rputs("</th>\n"
+					"  </tr>\n", r);
 				j = 0;
 			}
-				
 		}
 		
 		if ((j++) & 1)
@@ -174,7 +186,7 @@ static void list_songs(request_rec *r, struct mu_ent *p, mu_config *conf)
 			
 			if (local_options & MI_ALLOWSTREAM)	/* Display checkbox */
 				ap_rvputs(r, "    <input type=\"checkbox\" name=\"file\" value=\"",
-					q->file, "\" />\n", NULL);
+					(conf->options&MI_LISTCUSTOM)?q->uri:q->file, "\" />\n", NULL);
 			
 			if (!(conf->options & MI_LISTCUSTOM)) {
 				if (local_options & MI_ALLOWDWNLD)	/* Display [download] */
@@ -237,23 +249,31 @@ static void list_songs(request_rec *r, struct mu_ent *p, mu_config *conf)
 					break;
 				case SB_FILETYPE:
 					switch(q->filetype) {
+						case FT_FLAC:
+							ap_rputs("   <td>FLAC</td>\n", r);
+							break;
 						case FT_MP3:
-							ap_rvputs(r, "   <td>MP3</td>\n", NULL);
+							ap_rputs("   <td>MP3</td>\n", r);
 							break;
 						case FT_OGG:
-							ap_rvputs(r, "   <td>OGG</td>\n", NULL);
+							ap_rputs("   <td>Vorbis</td>\n", r);
+							break;
+						default:
+							ap_rputs("   <td></td>\n", r);
 							break;
 					}
 					break;
 				case SB_GENRE:
 					ap_rprintf(r, "   <td class=\"Genre\">%s</td>\n", q->genre ? ap_escape_html(r->pool, q->genre) : "");
 					break;
+				case SB_SIZE:
+					ap_rprintf(r, "   <td class=\"Size\">%ld</td>\n", q->size);
+					break;
 				default:
 					break;
 			}
 		}
 		ap_rputs("  </tr>\n", r);
-		q = q->next;
 	}
 
 }
@@ -266,8 +286,6 @@ static void list_songs(request_rec *r, struct mu_ent *p, mu_config *conf)
  * 
  * @param r Apache request_rec struct to handle connection details.
  * @param conf MusicIndex configuration paramaters struct.
- *
- * @return The corresponding option letter.
  */
 void treat_args(request_rec *r, mu_config *conf)
 {
@@ -282,38 +300,59 @@ void treat_args(request_rec *r, mu_config *conf)
 		p = ap_getword(r->pool, &s, '&');
 		if (!strncmp(p, "action=", 7)) {
 			p += 7;
-			if ((!strcmp(p, "Play+Selected")) && (conf->options & MI_ALLOWSTREAM)) {
-				conf->options |= MI_PLAYLIST;
-			} else if ((!strcmp(p, "playall") || !strcmp(p, "Play+All")) && (conf->options & MI_ALLOWSTREAM)) {
-				conf->options |= MI_PLAYALL; /* generate a playlist */
-			} else if (!strcmp(p, "Shuffle+All") && (conf->options & MI_ALLOWSTREAM)) {
-				conf->options |= MI_PLAYALL; /* generate a playlist */
-				conf->order[0] = SB_RANDOM;
-				conf->order[1] = SB_DEFAULT;
-				set_fctptrs(conf);
-			} else if (!strcmp(p, "Search") && (conf->options & MI_ALLOWSEARCH) && (conf->search[0] != '\0')) {
-				set_fctptrs(conf);
-				if (!conf->cache_path)
-					conf->options |= MI_QUICKPL;
-			} else if (!strcmp(p, "Recursive+Search") && (conf->options & MI_ALLOWSEARCH) && (conf->search[0] != '\0')) {
-				conf->play_recursive |= MI_RECURSIVE;
-				conf->order[0] = SB_DIR;
-				conf->order[1] = SB_URI;
-				set_fctptrs(conf);
-				if (!conf->cache_path)
-					conf->options |= MI_QUICKPL;
-			} else if (!strcmp(p, "Remove+from+Playlist")) {
-				conf->options |= MI_COOKIEDEL;
-			} else if (!strcmp(p, "Clear+Playlist")) {
-				conf->options |= MI_COOKIEPURGE;
-			} else if (!strcmp(p, "Stream+Playlist")) {
-				conf->options |= MI_COOKIESTREAM;
-			} else if (!strcmp(p, "Add+To+Playlist")) {
-				conf->options |= MI_COOKIEADD;
-			} else if (!strcmp(p, "Add+All+To+Playlist")) {
-				conf->options |= MI_COOKIEALL;
-				conf->play_recursive &= ~MI_RECURSIVE;
+			
+			if (conf->rss_items > 0) {
+				if (!strcmp(p, "RSS")) {
+					p += 3;
+					conf->options |= MI_RSS;
+					conf->order[0] = SB_MTIME;
+					set_fctptrs(conf);
+					return; /* if we're asked a RSS feed, nothing else should happen */
+				}
 			}
+			if (conf->options & MI_ALLOWSTREAM) {
+				if (!strcmp(p, "Play+Selected")) {
+					conf->options |= MI_STREAMLST;
+				} else if (!strcmp(p, "playall") || !strcmp(p, "Play+All")) {
+					conf->options |= MI_STREAMALL;
+				} else if (!strcmp(p, "Shuffle+All")) {
+					conf->options |= MI_STREAMALL;
+					conf->order[0] = SB_RANDOM;
+					conf->order[1] = SB_DEFAULT;
+					set_fctptrs(conf);
+				} else if (!strcmp(p, "Remove+from+Playlist")) {
+					conf->options |= MI_COOKIEDELLST;
+				} else if (!strcmp(p, "Clear+Playlist")) {
+					conf->options |= MI_COOKIEPURGE;
+				} else if (!strcmp(p, "Stream+Playlist")) {
+					conf->options |= MI_COOKIESTREAM;
+				} else if (!strcmp(p, "Add+To+Playlist")) {
+					conf->options |= MI_COOKIEADDLST;
+				} else if (!strcmp(p, "Add+All+To+Playlist")) {
+					conf->options |= MI_COOKIEADDALL;
+				}
+				
+				/* In any situation, no recursive directories
+				scan for Cookie operations */
+				if (conf->options & MI_COOKIEOP)
+					conf->play_recursive &= ~MI_RECURSIVE;
+			}
+			
+			if ((conf->options & MI_ALLOWSEARCH) && (conf->search != NULL) && (conf->search[0] != '\0')) {
+				if (!strcmp(p, "Search")) {
+					set_fctptrs(conf);
+					if (!conf->cache_path)
+						conf->options |= MI_QUICKPL;
+				} else if (!strcmp(p, "Recursive+Search")) {
+					conf->play_recursive |= MI_RECURSIVE;
+					conf->order[0] = SB_DIR;
+					conf->order[1] = SB_URI;
+					set_fctptrs(conf);
+					if (!conf->cache_path)
+						conf->options |= MI_QUICKPL;
+				}
+			}
+			
 			return;
 		}
 		else if (!strncmp(p, "sort=", 5)) {
@@ -354,6 +393,11 @@ void treat_args(request_rec *r, mu_config *conf)
 				ap_unescape_url(conf->search);
 			}
 		}
+		else if ((conf->rss_items > 0) && (!strncmp(p, "limit=", 6))) {
+			p += 6;
+			if (isdigit(*p))
+				conf->rss_items = atoi(p);
+		}
 	}
 }
 
@@ -375,10 +419,13 @@ static void send_url(request_rec *r, char *uri, char *command, mu_config *conf)
 	char *bufcoded, *decoded;
 	unsigned short l;
 	
-	strcpy(prefix, "http://");
-
-	/* if we have an icecast server, it takes precedence, in a 'staticdir' fashion */
-	if (conf->iceserver) {
+	/* If we have an icecast server, it takes precedence, in a 'staticdir' fashion.
+	If we're sending an RSS feed, it's up to us to send the ?stream links. */
+	if ((conf->iceserver) && !(conf->options & MI_RSS)) {
+		
+		/* icecast can only do http, so force that method */
+		strcpy(prefix, "http://");
+				
 		/* if we only have a port number, we assume icecast is running on the same host */
 		if (conf->iceserver[0] == ':')
 			strcat(prefix, r->hostname);
@@ -386,6 +433,9 @@ static void send_url(request_rec *r, char *uri, char *command, mu_config *conf)
 		strcat(prefix, conf->iceserver);
 	}
 	else {
+		strcpy(prefix, ap_http_method(r));
+ 		strcat(prefix, "://"); 
+
 		if (REQUEST_USER(r)) {
 			/* grab the auth credentials base64 encoded */
 			const char *auth = ap_table_get(r->headers_in, "Authorization");
@@ -403,9 +453,9 @@ static void send_url(request_rec *r, char *uri, char *command, mu_config *conf)
 		strcat(prefix, r->hostname);
 		
 		/* add the port number if needed */
-		if (r->server->port != 80) {
-			sprintf(str_port, "%u", r->server->port);
-			strcat(prefix, ":");
+		if ((strcmp(ap_http_method(r), "http") == 0 && ap_get_server_port(r) != DEFAULT_HTTP_PORT) ||
+		    (strcmp(ap_http_method(r), "https") == 0 && ap_get_server_port(r) != DEFAULT_HTTPS_PORT)) {
+			sprintf(str_port, ":%u", ap_get_server_port(r));
 			strcat(prefix, str_port);
 		}
 	}
@@ -413,8 +463,7 @@ static void send_url(request_rec *r, char *uri, char *command, mu_config *conf)
 	/* add the uri and potential command */
 	ap_rvputs(r, prefix, ap_escape_uri(r->pool, uri), NULL);
 	if (command)
-		ap_rvputs(r, command, NULL);
-	ap_rvputs(r, "\n", NULL);
+		ap_rputs(command, r);
 }	
 		
 /**
@@ -423,23 +472,22 @@ static void send_url(request_rec *r, char *uri, char *command, mu_config *conf)
  * This function takes care of the formating of the HTML page headers.
  * It sends the data that is common to all pages generated by the handler.
  * 
- * @bug No ending "/" on dir URLs.
- *
  * @param r Apache request_rec struct to handle connection details.
  * @param conf MusicIndex configuration paramaters struct.
  */
 void send_head(request_rec *r, mu_config *conf)
 {
-	char *s, *t, *u, bk;
-	request_rec *subreq;
-
-	s = ap_pstrdup(r->pool, r->uri);
+	char *uri, *dir, *u;
+	request_rec *subreq = NULL;
+#ifdef NEW_HEADER /* New ! :) */
+	mu_config *localconf = NULL;
+#endif
 
 	ap_rputs("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
-		"<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">\n"
-		"<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"en\" lang=\"en\">\n"
+		"<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.1//EN\" \"http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd\">\n"
+		"<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"en\">\n"
 		"<head>\n"
-		" <meta name=\"generator\" content=\"mod_musicindex\" />\n", r);
+		" <meta name=\"generator\" content=\"mod_musicindex/" MUSIC_VERSION_STRING "\" />\n", r);
 
 	if ((subreq = ap_sub_req_lookup_uri(conf->directory, r, NULL)) != NULL) {
 		DIR 		*dir;
@@ -487,70 +535,79 @@ void send_head(request_rec *r, mu_config *conf)
 	ap_rputs("\" />\n"
 		" </div>\n", r);
 
+	/* Display the box containing the path and the stream links */
 	ap_rputs(" <div id=\"maintitle\">\n"
 		"  <h1>\n", r);
 
-	u = s;
-	t = u + 1;
-	do{
+	uri = ap_pstrdup(r->pool, r->uri);
+
+#ifdef NEW_HEADER /* New ! :) */
+	for (u = uri; *u != '\0'; u++) {
+		dir = u;
 		while ((*u != '/') && (*u != '\0'))
 			u++;
 		
-		bk = *(++u);
-		*u ='\0';
+		if (conf->title == NULL) {
+			char bk = *(++u);
+			*u = '\0';
+			subreq = ap_sub_req_lookup_uri(uri, r, NULL);
+			*(u--) = bk;
+			localconf = (mu_config*)ap_get_module_config(subreq->per_dir_config, &musicindex_module);
+		}
+		else {
+			localconf = conf;
+			subreq = NULL;
+		}
 		
-#if 0 /* This code is here and works, it only displays directories in which the
-	 module is active... Only need to decide wether we use it or
-	 not... */
-	 /* XXX tv: I feel this will lead us & the user to headaches... */
-		subreq = ap_sub_req_lookup_uri(s, r, NULL);
-		
-		if (((mu_config*)ap_get_module_config(subreq->per_dir_config, &musicindex_module))->options & MI_ACTIVE) {
-			if (u == (s+1))
-				t = conf->title;
-			else
-				*(u-1) = '\0';
+		if ((localconf->options & MI_ACTIVE) || (localconf->title == NULL)) {
+			if ((u == uri) && (localconf->title != NULL))
+				dir = localconf->title;
 			
-			ap_rvputs(r, "   <a href=\"", s, "\">", t, "</a>\n", NULL);
-			*(u-1) = '/';
-			t = u;
+			*u = '\0';
+			ap_rvputs(r, "   <a href=\"", uri, "/\">", dir, "</a>\n", NULL);
+			*u = '/';
 			
-			if (*t)
-				ap_rvputs(r, "     <img src=\"", conf->directory, "/",
+			if (*(u+1) != '\0')
+				ap_rvputs(r, "   <img src=\"", conf->directory, "/",
 					conf->arrow, "\" alt=\"=>\" />\n", NULL);
 		}
-		*u = bk;
 		
-		ap_destroy_sub_req(subreq);
-#else
-		if (u == (s+1))
-			t = conf->title;
+		if (subreq != NULL)
+			ap_destroy_sub_req(subreq);
+	}
+#else /* Here to have a fallback solution if there is a problem with the new code */
+	for (u = uri; *u != '\0'; u++) {
+		dir = u;
+		while ((*u != '/') && (*u != '\0'))
+			u++;
+		
+		if (u == uri)
+			dir = conf->title;
 		else
-			*(u-1) = '\0';
-		
-		ap_rvputs(r, "   <a href=\"", s, "\">", t, "</a>\n", NULL);
-		*(u-1) = '/';	/* XXX ca ca marche pas, l'url se termine sans '/' final ce qui est mal */
-		t = u;
-		
-		*u = bk;
-
-		if (*t)
 			ap_rvputs(r, "     <img src=\"", conf->directory, "/",
 				conf->arrow, "\" alt=\"=>\" />\n", NULL);
+		
+		*u = '\0';
+		ap_rvputs(r, "   <a href=\"", uri, "/\">", dir, "</a>\n", NULL);
+		*u = '/';
+	}
 #endif
-	} while (*u != '\0');
 
 	ap_rputs("  </h1>\n", r);
 
 	if (conf->options & MI_ALLOWSTREAM) {
-		ap_rvputs(r, "  <span>\n"
-			"   <a class=\"shuffle\" "
+		ap_rvputs(r, "  <a class=\"shuffle\" "
 			"href=\"?option=recursive&amp;option=shuffle&amp;action=playall\">[",
 			_("Shuffle All"), "]</a>\n"
-			"   <a class=\"stream\" "
+			"  <a class=\"stream\" "
 			"href=\"?option=recursive&amp;action=playall\">[",
-			_("Stream All"), "]</a>\n"
-			"  </span>\n", NULL);
+			_("Stream All"), "]</a>\n", NULL);
+	}
+
+	if (conf->rss_items > 0) {
+		ap_rvputs(r, "    <a class=\"rss\" "
+			"href=\"?action=RSS\">[",
+			_("RSS"), "]</a>\n", NULL);
 	}
 	
 	ap_rputs(" </div>\n", r);
@@ -559,7 +616,8 @@ void send_head(request_rec *r, mu_config *conf)
  	if (conf->options & MI_ALLOWSEARCH){
 		ap_rvputs(r,
 			" <form method=\"get\" action=\"", ap_escape_uri(r->pool, r->uri), "\""
-			"enctype=\"application/x-www-form-urlencoded\" id=\"searching\">\n"
+				" enctype=\"application/x-www-form-urlencoded\""
+				" id=\"searching\">\n"
 			"  <p>\n"
 			"   <input type=\"text\" name=\"option\" />\n"
 			"   <br />\n"
@@ -590,29 +648,23 @@ void send_head(request_rec *r, mu_config *conf)
  */
 void send_directories(request_rec *r, struct mu_ent *p, mu_config *conf)
 {
-	struct mu_ent *q = p;
+	struct mu_ent *q;
 	unsigned short dircnt = 0, nb = 0;
 	char *c;
 	char temp[MAX_STRING];
 
-	if (!q)
-		return;
-
-	if (q->filetype != FT_DIR)
-		return;
-	
 	/* We count the number of directories for later use */
-	do {
+	for (q=p; (q != NULL) && (q->filetype == FT_DIR); q = q->next)
 		nb++;
-	} while ((q = q->next) && (q->filetype == FT_DIR));
 	
-	q = p;
-		
+	if (nb == 0)
+		return;
+	
 	ap_rputs("<!-- begin subdirs -->\n<h2>",r);
 	ap_rprintf(r, _("Music Directories (%d)"), nb);
 	ap_rputs("</h2>\n\n<table id=\"directories\">\n", r);
 
-	while (q && (q->filetype == FT_DIR)) {
+	for (q=p; (q != NULL) && (q->filetype == FT_DIR); q = q->next) {
 		c = ap_cpystrn(temp, q->file, MAX_STRING)-1;
 		*c = '\0';
 		
@@ -648,15 +700,22 @@ That's IMHO a bad feature */
 	    		temp, "</a><br />\n", NULL);
 		
 		if (conf->options & MI_ALLOWSTREAM) {
-			ap_rvputs(r, "     <a class=\"shuffle\" href=\"",
+			ap_rvputs(r, "    <a class=\"shuffle\" href=\"",
 				ap_escape_uri(r->pool, q->file),
 				"?option=recursive&amp;option=shuffle&amp;action=playall\">[",
 				_("Shuffle"), "]</a>\n", NULL);
 			
-			ap_rvputs(r, "     <a class=\"stream\" href=\"",
+			ap_rvputs(r, "    <a class=\"stream\" href=\"",
 				ap_escape_uri(r->pool, q->file),
 				"?option=recursive&amp;action=playall\">[",
-				_("Stream"), "]</a>\n", NULL);
+				_("Stream"), "]</a>\n", NULL);		
+		}
+		
+		if (conf->rss_items > 0) {
+			ap_rvputs(r, "    <a class=\"rss\" href=\"",
+				ap_escape_uri(r->pool, q->file),
+				"?action=RSS\">[",
+				_("RSS"), "]</a>\n", NULL);
 		}
 		
 		ap_rputs("   </div>\n"
@@ -666,7 +725,6 @@ That's IMHO a bad feature */
 			dircnt = 0;
 			ap_rputs(" </tr>\n", r);
 		}
-		q = q->next;
 	}
 	
 	if (dircnt != 0)
@@ -682,15 +740,13 @@ That's IMHO a bad feature */
  * list (if any) of the available songs in the current p list.
  * Depending on config options, it allows streaming/shuffle etc.
  *
- * @todo better handling of per-directory configuration for select panel
- * 
  * @param r Apache request_rec struct to handle connection details.
  * @param p struct mu_ent.
  * @param conf MusicIndex configuration paramaters struct.
  */
 void send_tracks(request_rec *r, struct mu_ent *p, mu_config *conf)
 {
-	struct mu_ent *q = p;
+	struct mu_ent *q;
 	unsigned short nb = 0;
 	
 	/* We count the number of songs for later use */
@@ -699,13 +755,7 @@ void send_tracks(request_rec *r, struct mu_ent *p, mu_config *conf)
 			nb++;
 	}
 
-	/* Go to the first song entry (just after dirs) */
-	for (q=p; (q!=NULL); q = q->next) {
-		if (q->filetype != FT_DIR)
-			break;
-	}
-
-	if (!(q))
+	if (nb == 0)
 		return;
 
 	ap_rputs("<!-- begin tracks -->\n<h2>", r);
@@ -722,10 +772,9 @@ void send_tracks(request_rec *r, struct mu_ent *p, mu_config *conf)
 	
 	ap_rputs(" <table>\n", r);
 
-	list_songs(r, q, conf);
+	list_songs(r, p, conf);
 	
-	/* Do NOT show 'select all' if there's nothing to select... TODO: better
-	   handling of per-directory configuration */
+	/* Do NOT show 'select all' if there's nothing to select... */
 	if ((conf->search) && (conf->options & MI_ALLOWSTREAM))
 		ap_rputs("  <tr class=\"title\"><th align=\"left\" colspan=\"10\">\n"
 			"   <input type=\"checkbox\" name=\"all\" onClick=\"for(var i=0;i<this.form.elements.length;i++){var inpt=this.form.elements[i];"
@@ -755,9 +804,21 @@ void send_tracks(request_rec *r, struct mu_ent *p, mu_config *conf)
 		"<!-- end tracks -->\n\n", r);
 }
 
+/**
+ * Sends track listing for the current custom playlist.
+ *
+ * This function takes care of preparing and sending to the client the
+ * list (if any) of the available songs in the current custom p list,
+ * retrieved from a cookie.
+ * Depending on config options, it allows streaming/shuffle etc.
+ * 
+ * @param r Apache request_rec struct to handle connection details.
+ * @param p struct mu_ent.
+ * @param conf MusicIndex configuration paramaters struct.
+ */
 void send_customlist(request_rec *r, mu_ent *p, mu_config *conf)
 {
-	struct mu_ent *q = p;
+	struct mu_ent *q;
 	unsigned short nb = 0;
 	
 	if (p == NULL)
@@ -768,7 +829,8 @@ void send_customlist(request_rec *r, mu_ent *p, mu_config *conf)
 		nb++;
 	}
 
-	ap_rputs("<!-- begin custom -->\n<h2>", r);
+	ap_rputs("<!-- begin custom -->\n"
+		"<h2>", r);
 	ap_rprintf(r, _("Custom Playlist (%d)"), nb);
 	ap_rputs("</h2>\n\n", r);
 	
@@ -810,7 +872,7 @@ void send_playlist(request_rec *r, mu_ent *p, mu_config *conf)
 
 	ap_rputs("#EXTM3U\n", r);
 
-	while (q)
+	for (q=p; q!=NULL; q=q->next)
 	{
 		ap_rprintf(r, "#EXTINF:%li,", q->length);
 		if (q->artist)
@@ -818,12 +880,84 @@ void send_playlist(request_rec *r, mu_ent *p, mu_config *conf)
 		ap_rvputs(r, q->title, NULL);
 		if (q->album)
 			ap_rvputs(r, " (", q->album, ")", NULL);
-		ap_rvputs(r, "\n", NULL);
+		ap_rputc('\n', r);
 		send_url(r, q->uri, NULL, conf);
-		q = q->next;
+		ap_rputc('\n', r);
 	}
 }
 
+/**
+ * Sends RSS feed.
+ *
+ * see http://blogs.law.harvard.edu/tech/rss
+ * and http://feedvalidator.org/docs/rss2.html
+ *
+ * @param r Apache request_rec struct to handle connection details.
+ * @param conf MusicIndex configuration paramaters struct.
+ */
+void send_rss(request_rec *r, mu_ent *p, mu_config *conf)
+{
+	mu_ent *q;
+	unsigned short limit = conf->rss_items;
+	
+	if (!p)
+		return;
+	
+	ap_rvputs(r, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
+		"<rss version=\"2.0\">\n"
+		" <channel>\n"
+		"  <title>RSS Feed for ", r->uri, "</title>\n"
+		"  <link>", NULL);
+	send_url(r, r->uri, NULL, conf);
+	ap_rputs("</link>\n", r);
+	ap_rprintf(r, "  <description>%d most recent songs from %s</description>\n", conf->rss_items, r->uri);
+	ap_rputs("  <generator>mod_musicindex/" MUSIC_VERSION_STRING "</generator>\n"
+		"  <docs>http://backend.userland.com/rss</docs>\n"
+		"  <ttl>60</ttl>\n",
+		r);
+	
+	for (q=p; limit && q; q=q->next) {
+		if (q->filetype == FT_DIR)
+			continue;
+		
+		limit--;
+		
+		ap_rvputs(r, "  <item>\n"
+			"   <title>", ap_escape_html(r->pool, q->title), "</title>\n", NULL);
+	
+		if (conf->options & MI_ALLOWSTREAM) {
+			ap_rputs("   <link>", r);
+			send_url(r, q->uri, "?stream", conf);
+			ap_rputs("</link>\n", r);
+		}
+#if 0  /* disabled atm */
+		if (conf->options & MI_ALLOWDWNLD)
+			ap_rvputs(r, "   <enclosure url=\"", ap_escape_html(r->pool, ap_escape_uri(r->pool, q->file)), "\" length=\"",
+					q->size, "\" type=\"audio/mpeg\" />\n",   NULL); /* XXX being able to index mime_handlers with q->filetype would be great! */
+#endif		
+		ap_rputs("   <description>\n", r);
+		if (q->artist)
+			ap_rprintf(r, "    Artist: %s |\n", ap_escape_html(r->pool, q->artist));
+		if (q->album)
+			ap_rprintf(r, "    Album: %s |\n", ap_escape_html(r->pool, q->album));
+		if (q->track)
+			ap_rprintf(r, "    Track: %u |\n", q->track);
+		if (q->posn)
+			ap_rprintf(r, "    Disc: %u |\n", q->posn);
+		if (q->length)
+			ap_rprintf(r, "    Length: %lu:%.2lu |\n", q->length / 60, q->length % 60);
+		if (q->genre)
+			ap_rprintf(r, "    Genre: %s |\n", ap_escape_html(r->pool, q->genre));
+		if (q->bitrate)
+			ap_rprintf(r, "    Bitrate: %lu\n", q->bitrate >> 10);
+		ap_rputs("   </description>\n"
+			"  </item>\n", r);
+	}
+	ap_rputs(" </channel>\n"
+		"</rss>\n", r);
+
+}
+
 /**
  * Sends HTML page footers.
  *
@@ -836,25 +970,32 @@ void send_playlist(request_rec *r, mu_ent *p, mu_config *conf)
  */
 void send_foot(request_rec *r, mu_config *conf)
 {
-	ap_rprintf(r, "<!-- begin footer -->\n<!-- mod_musicindex v.%s -->\n", MUSIC_VERSION_STRING);
-	ap_rprintf(r, "<!-- Authors: %s -->\n", MUSIC_AUTHORS_STRING);
+	ap_rputs("<!-- begin footer -->\n"
+		"<!-- mod_musicindex v." MUSIC_VERSION_STRING " -->\n"
+		"<!-- Authors: " MUSIC_AUTHORS_STRING " -->\n", r);
 	ap_rputs("<div id=\"footer\">\n"
-		" <div id=\"valid\">\n"
-#if 1
-		"  <a href=\"http://validator.w3.org/check/referer\">\n"
-#else
-		"  <a href=\"http://localhost/w3c-markup-validator/check/referer\">\n"
-#endif
-		"   <img src=\"http://www.w3.org/Icons/valid-xhtml10\"\n"
-        	"    alt=\"Valid XHTML 1.0!\" height=\"31\" width=\"88\" />\n"
-		"  </a>\n"
-		"  <a href=\"http://jigsaw.w3.org/css-validator/\">\n"
+		" <div id=\"valid\">\n", r);
+	/* Should be possible to use on-site validation icons */
+	ap_rputs("  <a href=\"http://validator.w3.org/check?uri=referer\">\n"
+		"   <img src=\"http://www.w3.org/Icons/valid-xhtml11\"\n"
+        	"    alt=\"Valid XHTML 1.1!\" height=\"31\" width=\"88\" />\n"
+		"  </a>\n", r);
+	ap_rputs("  <a href=\"http://jigsaw.w3.org/css-validator/check/referer\">\n"
 		"   <img src=\"http://jigsaw.w3.org/css-validator/images/vcss\"\n"
 		"    alt=\"Valid CSS!\" height=\"31\" width=\"88\" />\n"
-		"  </a>\n"
-		" </div>\n", r);
-	ap_rprintf(r, " <div id=\"name\">"
+		"  </a>\n", r);
+#if 1 /* Is it really useful ? */
+	ap_rputs("  <a href=\"http://rss.scripting.com/\">\n"
+		"   <img src=\"http://cyber.law.harvard.edu/blogs/gems/tech/loveRss.gif\"\n"
+		"    width=\"65\" height=\"59\" alt=\"Valid RSS feed.\" />\n"
+		"  </a>\n", r);
+#endif
+	ap_rputs(" </div>\n"
+		" <div id=\"name\">"
 		"<a href=\"http://freshmeat.net/projects/musicindex/\">"
-		"MusicIndex v.%s</a></div>\n", MUSIC_VERSION_STRING);
-	ap_rputs("</div>\n<!-- end footer -->\n\n</body>\n</html>", r);
+		"MusicIndex v." MUSIC_VERSION_STRING "</a></div>\n"
+		"</div>\n", r);
+	ap_rputs("<!-- end footer -->\n\n"
+		"</body>\n"
+		"</html>", r);
 }
